@@ -442,7 +442,7 @@ static int snd_pcm_ioplug_start(snd_pcm_t *pcm)
 	int err;
 	
 	if (io->data->state != SND_PCM_STATE_PREPARED)
-		return -EBUSY;
+		return -EBADFD;
 
 	err = io->data->callback->start(io->data);
 	if (err < 0)
@@ -625,7 +625,7 @@ static snd_pcm_sframes_t snd_pcm_ioplug_avail_update(snd_pcm_t *pcm)
 	snd_pcm_uframes_t avail;
 
 	snd_pcm_ioplug_hw_ptr_update(pcm);
-	if (io->data->state == SNDRV_PCM_STATE_XRUN)
+	if (io->data->state == SND_PCM_STATE_XRUN)
 		return -EPIPE;
 	if (pcm->stream == SND_PCM_STREAM_CAPTURE &&
 	    pcm->access != SND_PCM_ACCESS_RW_INTERLEAVED &&
@@ -710,6 +710,36 @@ static int snd_pcm_ioplug_munmap(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
 	return 0;
 }
 
+static snd_pcm_chmap_query_t **snd_pcm_ioplug_query_chmaps(snd_pcm_t *pcm)
+{
+	ioplug_priv_t *io = pcm->private_data;
+
+	if (io->data->version >= 0x010002 &&
+	    io->data->callback->query_chmaps)
+		return io->data->callback->query_chmaps(io->data);
+	return NULL;
+}
+
+static snd_pcm_chmap_t *snd_pcm_ioplug_get_chmap(snd_pcm_t *pcm)
+{
+	ioplug_priv_t *io = pcm->private_data;
+
+	if (io->data->version >= 0x010002 &&
+	    io->data->callback->get_chmap)
+		return io->data->callback->get_chmap(io->data);
+	return NULL;
+}
+
+static int snd_pcm_ioplug_set_chmap(snd_pcm_t *pcm, const snd_pcm_chmap_t *map)
+{
+	ioplug_priv_t *io = pcm->private_data;
+
+	if (io->data->version >= 0x010002 &&
+	    io->data->callback->set_chmap)
+		return io->data->callback->set_chmap(io->data, map);
+	return -ENXIO;
+}
+
 static void snd_pcm_ioplug_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	ioplug_priv_t *io = pcm->private_data;
@@ -760,6 +790,9 @@ static const snd_pcm_ops_t snd_pcm_ioplug_ops = {
 	.dump = snd_pcm_ioplug_dump,
 	.mmap = snd_pcm_ioplug_mmap,
 	.munmap = snd_pcm_ioplug_munmap,
+	.query_chmaps = snd_pcm_ioplug_query_chmaps,
+	.get_chmap = snd_pcm_ioplug_get_chmap,
+	.set_chmap = snd_pcm_ioplug_set_chmap,
 };
 
 static const snd_pcm_fast_ops_t snd_pcm_ioplug_fast_ops = {
@@ -877,7 +910,7 @@ callback.
 Finally, the dump callback is used to print the status of the plugin.
 
 The hw_params constraints can be defined via either
-#snd_pcm_iplug_set_param_minmax() and #snd_pcm_ioplug_set_param_list()
+#snd_pcm_ioplug_set_param_minmax() and #snd_pcm_ioplug_set_param_list()
 functions after calling #snd_pcm_ioplug_create().
 The former defines the minimal and maximal acceptable values for the
 given hw_params parameter (SND_PCM_IOPLUG_HW_XXX).
@@ -917,7 +950,8 @@ int snd_pcm_ioplug_create(snd_pcm_ioplug_t *ioplug, const char *name,
 	/* We support 1.0.0 to current */
 	if (ioplug->version < 0x010000 ||
 	    ioplug->version > SND_PCM_IOPLUG_VERSION) {
-		SNDERR("ioplug: Plugin version mismatch\n");
+		SNDERR("ioplug: Plugin version mismatch: 0x%x\n",
+		       ioplug->version);
 		return -ENXIO;
 	}
 
